@@ -37,20 +37,22 @@
 class DataStructurePrivate
 {
 public:
-    DataStructurePrivate() {}
+    DataStructurePrivate()
+        : _engine(0)
+    {
+    }
 
     /**
      * self pointer to DataStructure
      */
     boost::weak_ptr<DataStructure> q;
 
-    QMap<int, DataList> _dataTypeLists;         // list if data elements associated to specific type
-    QMap<int, bool> _dataTypeVisibility;
-    QMap<int, PointerList> _pointerTypeLists;   // list of pointers associated to specific type
-    QMap<int, bool> _pointerTypeVisibility;
-
     int _identifierCount;   // represents the next identifier that will be assigend to data/pointer
 
+    QMap<int, DataList> _dataTypeLists;         // list if data elements associated to specific type
+    QMap<int, PointerList> _pointerTypeLists;   // list of pointers associated to specific type
+    QHash<int, DataPtr> _dataIdentifierMap;
+
     QList<GroupPtr> _groups;
 
     QPointF _relativeCenter;
@@ -60,8 +62,6 @@ public:
 
     QScriptValue _value;
     QScriptEngine *_engine;
-    QMap<QString, QVariant> m_globalPropertiesData;
-    QMap<QString, QVariant> m_globalPropertiesPointer;
 };
 
 
@@ -83,7 +83,6 @@ void DataStructure::setQpointer(DataStructurePtr q)
 void DataStructure::initialize()
 {
     d->_readOnly = false;
-    updateRelativeCenter();
     d->_identifierCount = 1;
 
     // create type lists
@@ -107,24 +106,22 @@ DataStructurePtr DataStructure::getDataStructure() const
     return px;
 }
 
-DataStructure::DataStructure(Document *parent) :
-    d(new DataStructurePrivate)
+DataStructure::DataStructure(Document *parent)
+    : d(new DataStructurePrivate)
 {
     d->_document = parent;
-    connect(this, SIGNAL(changed()), parent, SLOT(resizeDocumentIncrease()));
-    connect(this, SIGNAL(resizeRequest(Document::Border)), parent, SLOT(resizeDocumentBorder(Document::Border)));
     // further initialization is done by separate call to initialize()
 }
 
 void DataStructure::importStructure(DataStructurePtr other)
 {
     d->_readOnly = other->readOnly();
-    updateRelativeCenter();
     // FIXME set dynamic properties
 
     QHash <Data*, DataPtr> dataTodata;
-    foreach(DataPtr n, other->dataList()) {
-        DataPtr newdata = addData(n->property("name").toString());
+    //FIXME only reading default of elements
+    foreach(DataPtr n, other->dataList(0)) {
+        DataPtr newdata = createData(n->property("name").toString(), 0);
         newdata->setColor(n->color());
         //FIXME all dynamic properties must be set
         newdata->setX(n->x());
@@ -132,18 +129,15 @@ void DataStructure::importStructure(DataStructurePtr other)
         newdata->setWidth(n->width());
         dataTodata.insert(n.get(), newdata);
     }
-    foreach(PointerPtr e, other->pointers()) {
+    //FIXME only import of default elements
+    foreach(PointerPtr e, other->pointers(0)) {
         DataPtr from =  dataTodata.value(e->from().get());
         DataPtr to =  dataTodata.value(e->to().get());
 
-        PointerPtr newPointer = addPointer(from, to);
+        PointerPtr newPointer = createPointer(from, to, 0);
         newPointer->setColor(e->color());
         //FIXME all dynamic properties must be set
     }
-
-    connect(this, SIGNAL(changed()), d->_document, SLOT(resizeDocumentIncrease()));
-    connect(this, SIGNAL(resizeRequest(Document::Border)), d->_document, SLOT(resizeDocumentBorder(Document::Border)));
-    emit changed();
 }
 
 
@@ -203,7 +197,6 @@ void DataStructure::registerDataType(int identifier)
         return;
     }
     d->_dataTypeLists.insert(identifier, DataList());
-    d->_dataTypeVisibility.insert(identifier, true);
 }
 
 
@@ -218,7 +211,6 @@ void DataStructure::registerPointerType(int identifier)
         return;
     }
     d->_pointerTypeLists.insert(identifier, PointerList());
-    d->_pointerTypeVisibility.insert(identifier, true);
 }
 
 
@@ -234,7 +226,6 @@ void DataStructure::removeDataType(int identifier)
     }
     d->_dataTypeLists[identifier].clear();
     d->_dataTypeLists.remove(identifier);
-    d->_dataTypeVisibility.remove(identifier);
 }
 
 
@@ -250,19 +241,6 @@ void DataStructure::removePointerType(int pointerType)
     }
     d->_pointerTypeLists[pointerType].clear();
     d->_pointerTypeLists.remove(pointerType);
-    d->_pointerTypeVisibility.remove(pointerType);
-}
-
-
-bool DataStructure::isDataVisible(int dataType) const
-{
-    return d->_dataTypeVisibility.value(dataType);
-}
-
-
-bool DataStructure::isPointerVisible(int pointerType) const
-{
-    return d->_pointerTypeVisibility.value(pointerType);
 }
 
 
@@ -309,6 +287,7 @@ void DataStructure::remove()
         }
     }
     d->_dataTypeLists.clear();
+    d->_dataIdentifierMap.clear();
 
     d->_document->remove(getDataStructure());
 }
@@ -318,61 +297,70 @@ int DataStructure::generateUniqueIdentifier()
     return d->_identifierCount++;
 }
 
-DataPtr DataStructure::addData(const QString& name, int dataType)
+DataPtr DataStructure::createData(const QString& name, int dataType)
 {
     if (d->_readOnly) {
         return DataPtr();
     }
-
     DataPtr n = Data::create(this->getDataStructure(), generateUniqueIdentifier(), dataType);
     n->setProperty("name", name);
 
-    return addData(n, dataType);
+    return addData(n);
 }
 
-DataPtr DataStructure::addData(DataPtr data, int dataType)
+DataPtr DataStructure::addData(DataPtr data)
 {
-    Q_ASSERT(dataType >= 0 && dataType <= d->_dataTypeLists.size());
+    Q_ASSERT(data->dataType() >= 0 && data->dataType() <= d->_dataTypeLists.size());
 
-    d->_dataTypeLists[dataType].append(data);
-    QMap<QString, QVariant>::const_iterator i = d->m_globalPropertiesData.constBegin();
-    while (i != d->m_globalPropertiesData.constEnd()) {
-        data->addDynamicProperty(i.key(), i.value());
-        ++i;
+    // set data type properties
+    d->_dataTypeLists[data->dataType()].append(data);
+    DataTypePtr type = document()->dataType(data->dataType());
+    foreach(const QString &property, type->properties()) {
+        if (!data->property(property.toLatin1()).isValid()
+            || data->property(property.toLatin1()).isNull())
+        {
+            data->addDynamicProperty(property, type->propertyDefaultValue(property));
+        }
     }
+
+    // register data at fast access identifier hash
+    d->_dataIdentifierMap.insert(data->identifier(), data);
+
     emit dataCreated(data);
     emit changed();
 
     connect(data.get(), SIGNAL(propertyChanged(QString)),     this, SIGNAL(changed()));
     connect(data.get(), SIGNAL(colorChanged(QColor)),         this, SIGNAL(changed()));
-    connect(data.get(), SIGNAL(posChanged(QPointF)),          this, SIGNAL(changed()));
+    connect(data.get(), SIGNAL(posChanged(QPointF)),          this, SIGNAL(dataPositionChanged(QPointF)));
     connect(data.get(), SIGNAL(useColorChanged(bool)),        this, SIGNAL(changed()));
     return data;
 }
 
-DataList DataStructure::addDataList(DataList dataList, int dataType)
+DataPtr DataStructure::createData(const QString& name, const QPointF& pos, int dataType)
 {
-    Q_ASSERT(dataType >= 0 && dataType < d->_dataTypeLists.size());
-
-    foreach(DataPtr n, dataList) {
-        d->_dataTypeLists[dataType].append(n);
-        emit dataCreated(n);
-
-        connect(n.get(), SIGNAL(propertyChanged(QString)),     this, SIGNAL(changed()));
-        connect(n.get(), SIGNAL(colorChanged(QColor)),         this, SIGNAL(changed()));
-        connect(n.get(), SIGNAL(posChanged(QPointF)),          this, SIGNAL(changed()));
-        connect(n.get(), SIGNAL(useColorChanged(bool)),        this, SIGNAL(changed()));
+    if (DataPtr data = createData(name, dataType)) {
+        data->setPos(pos.x(), pos.y());
+        return data;
     }
-    emit changed();
+    return DataPtr();
+}
+
+DataList DataStructure::addDataList(DataList dataList)
+{
+    foreach(DataPtr n, dataList) {
+        addData(n);
+    }
+
     return dataList;
 }
 
+
 DataList DataStructure::addDataList(QList< QPair<QString, QPointF> > dataList, int dataType)
 {
     QList< DataPtr > dataCreateList;
     QList< QPair<QString, QPointF> >::const_iterator dataDefinition = dataList.constBegin();
     while (dataDefinition != dataList.constEnd()) {
-        if (DataPtr data = addData(dataDefinition->first, dataType)) {
+        if (DataPtr data = createData(dataDefinition->first, dataType)) {
             data->setPos(dataDefinition->second.x(), dataDefinition->second.y());
             dataCreateList << data;
         }
@@ -381,33 +369,28 @@ DataList DataStructure::addDataList(QList< QPair<QString, QPointF> > dataList, i
     return dataCreateList;
 }
 
-
-PointerPtr DataStructure::addPointer(PointerPtr pointer, int pointerType)
+PointerPtr DataStructure::addPointer(PointerPtr pointer)
 {
-    Q_ASSERT(d->_pointerTypeLists.contains(pointerType));
+    Q_ASSERT(d->_pointerTypeLists.contains(pointer->pointerType()));
 
-    d->_pointerTypeLists[pointerType].append(pointer);
-    QMap<QString, QVariant>::const_iterator i = d->m_globalPropertiesPointer.constBegin();
-    while (i != d->m_globalPropertiesPointer.constEnd()) {
-        pointer->addDynamicProperty(i.key(), i.value());
-        ++i;
+    // set pointer type properties
+    d->_pointerTypeLists[pointer->pointerType()].append(pointer);
+    PointerTypePtr type = document()->pointerType(pointer->pointerType());
+    foreach(const QString &property, type->properties()) {
+        if (!pointer->property(property.toLatin1()).isValid()
+            || pointer->property(property.toLatin1()).isNull())
+        {
+            pointer->addDynamicProperty(property, type->propertyDefaultValue(property));
+        }
     }
+
     emit pointerCreated(pointer);
     emit changed();
     connect(pointer.get(), SIGNAL(changed()), this, SIGNAL(changed()));
     return pointer;
 }
 
-DataPtr DataStructure::addData(const QString& name, const QPointF& pos, int dataType)
-{
-    if (DataPtr data = addData(name, dataType)) {
-        data->setPos(pos.x(), pos.y());
-        return data;
-    }
-    return DataPtr();
-}
-
-PointerPtr DataStructure::addPointer(DataPtr from, DataPtr to, int pointerType)
+PointerPtr DataStructure::createPointer(DataPtr from, DataPtr to, int pointerType)
 {
     Q_ASSERT(d->_document->pointerTypeList().contains(pointerType));
 
@@ -422,14 +405,19 @@ PointerPtr DataStructure::addPointer(DataPtr from, DataPtr to, int pointerType)
         return PointerPtr();
     }
     PointerPtr pointer = Pointer::create(getDataStructure(), from, to, pointerType);
-    return addPointer(pointer, pointerType);
+    return addPointer(pointer);
 }
 
 DataPtr DataStructure::getData(int uniqueIdentifier)
 {
-    foreach(const DataList& dataType, d->_dataTypeLists) {
+    if (d->_dataIdentifierMap.contains(uniqueIdentifier)) {
+        return d->_dataIdentifierMap[uniqueIdentifier];
+    }
+
+    foreach(const DataList &dataType, d->_dataTypeLists) {
         foreach(DataPtr data, dataType) {
             if (data->identifier() == uniqueIdentifier) {
+                kWarning() << "Access do data element that is not registered at data-identifier mapper.";
                 return data;
             }
         }
@@ -439,42 +427,33 @@ DataPtr DataStructure::getData(int uniqueIdentifier)
 
 void DataStructure::remove(DataPtr data)
 {
-    //Note: important for resize: remove node before emit resizeRequest
-    Document *doc = DocumentManager::self().activeDocument();
-
-    if (doc != 0) {
-        qreal xCenter = (doc->left() + doc->right() )/2;
-        qreal yCenter = (doc->top() + doc->bottom() )/2;
-
-        if (data->x() < xCenter) emit resizeRequest(Document::BorderLeft);
-        if (data->x() > xCenter) emit resizeRequest(Document::BorderRight);
-        if (data->y() < yCenter) emit resizeRequest(Document::BorderTop);
-        if (data->y() > yCenter) emit resizeRequest(Document::BorderBottom);
+    if (!d->_dataTypeLists[data->dataType()].contains(data)) {
+        kWarning() << "Data element not registered, aborting removal.";
+        return;
     }
 
-    // remove from internal list
-    QMap<int,DataList>::iterator iter = d->_dataTypeLists.begin();
-    while (iter != d->_dataTypeLists.end()) {
-        if (iter->removeOne(data)) {
-            // only remove data element if it is registered
-            data->remove();
-        }
-        ++iter;
+    // remove from internal lists
+    if (d->_dataIdentifierMap.remove(data->identifier()) != 1) {
+        kWarning() << "Data identifier hash is dirty.";
+    }
+    if (d->_dataTypeLists[data->dataType()].removeOne(data)) {
+        // only remove data element if it is registered
+        emit dataPositionChanged(QPointF(data->x(), data->y()));
+        data->remove();
     }
-    updateRelativeCenter();
     emit changed();
 }
 
 void DataStructure::remove(PointerPtr pointer)
 {
+    if (!d->_pointerTypeLists[pointer->pointerType()].contains(pointer)) {
+        kWarning() << "Pointer not registered, aborting removal.";
+        return;
+    }
     // remove from internal list
-    QMap<int,PointerList>::iterator iter = d->_pointerTypeLists.begin();
-    while (iter != d->_pointerTypeLists.end()) {
-        if (iter->removeOne(pointer)) {
-            // only remove pointer if it is registered
-            pointer->remove();
-        }
-        ++iter;
+    if (d->_pointerTypeLists[pointer->pointerType()].removeOne(pointer)) {
+        // only remove pointer if it is registered
+        pointer->remove();
     }
     emit changed();
 }
@@ -493,18 +472,6 @@ GroupPtr DataStructure::addGroup(const QString& name)
     return group;
 }
 
-void DataStructure::updateRelativeCenter()
-{
-    if (parent() != 0) {
-        Document *gd = qobject_cast<Document*>(parent());
-        d->_relativeCenter.setY((gd->bottom() + gd->top()) / 2);
-        d->_relativeCenter.setX((gd->right() + gd->left()) / 2);
-    } else {
-        d->_relativeCenter.setY(0);
-        d->_relativeCenter.setX(0);
-    }
-}
-
 void DataStructure::setName(const QString& s)
 {
     d->_name = s;
@@ -531,8 +498,8 @@ void DataStructure::renameDynamicProperty(const QString& oldName, const QString&
         kWarning() << "Property identifier \"" << newName << "\" is not valid: aborting";
         return;
     }
-    setProperty(newName.toStdString().c_str(), property(oldName.toStdString().c_str()));
-    setProperty(oldName.toStdString().c_str(), QVariant::Invalid);
+    setProperty(newName.toLatin1(), property(oldName.toLatin1()));
+    setProperty(oldName.toLatin1(), QVariant::Invalid);
 }
 
 void DataStructure::add_property(const QString& name, const QVariant& value)
@@ -545,77 +512,6 @@ void DataStructure::remove_property (const QString& name)
     removeDynamicProperty(name);
 }
 
-void DataStructure::addPointersDynamicProperty(const QString& property, const QVariant& value)
-{
-    // do not change properties concurrently, not thread safe
-    foreach(const PointerList& pointerType, d->_pointerTypeLists) {
-        PointerList::const_iterator pointer = pointerType.constBegin();
-        while (pointer != pointerType.constEnd()) {
-            (*pointer)->addDynamicProperty(property, value);
-            ++pointer;
-        }
-    }
-    d->m_globalPropertiesData.insert(property, value);
-}
-
-void DataStructure::removePointersDynamicProperty(const QString& property)
-{
-    // do not change properties concurrently, not thread safe
-    foreach(const PointerList& pointerType, d->_pointerTypeLists) {
-        PointerList::const_iterator pointer = pointerType.constBegin();
-        while (pointer != pointerType.constEnd()) {
-            (*pointer)->removeDynamicProperty(property);
-            ++pointer;
-        }
-    }
-    d->m_globalPropertiesData.remove(property);
-}
-
-
-void DataStructure::setDataColor(const QColor& color, int dataType)
-{
-    QtConcurrent::blockingMap(d->_dataTypeLists[dataType], DataColorSetted(color));
-}
-
-
-void DataStructure::setPointerColor(const QColor& color, int pointerType)
-{
-    QtConcurrent::blockingMap(d->_pointerTypeLists[pointerType], PointerColorSetted(color));
-}
-
-
-void DataStructure::setDataVisibility(bool visible, int dataType)
-{
-    d->_dataTypeVisibility[dataType] = visible;
-    // set visibility of data elements
-    QtConcurrent::blockingMap(d->_dataTypeLists[dataType], DataVisibilitySetted(visible));
-    foreach(DataPtr data, dataList(dataType)) {
-        foreach(PointerPtr pointer, data->pointerList()) {
-            pointer->setVisible(visible);
-        }
-    }
-}
-
-
-void DataStructure::toggleDataVisibility(int dataType)
-{
-    setDataVisibility(!isDataVisible(dataType), dataType);
-}
-
-
-void DataStructure::setPointerVisibility(bool visible, int pointerType)
-{
-    d->_pointerTypeVisibility[pointerType] = visible;
-    QtConcurrent::blockingMap(d->_pointerTypeLists[pointerType], PointerVisibilitySetted(visible));
-}
-
-
-void DataStructure::togglePointerVisibility(int pointerType)
-{
-    setPointerVisibility(!isPointerVisible(pointerType), pointerType);
-}
-
-
 void DataStructure::setEngine(QScriptEngine *engine)
 {
     d-> _engine = engine;
@@ -657,17 +553,11 @@ bool DataStructure::readOnly() const
     return d->_readOnly;
 }
 
-
 QString DataStructure::name() const
 {
     return d->_name;
 }
 
-QPointF DataStructure::relativeCenter() const
-{
-    return d->_relativeCenter;
-}
-
 QScriptValue DataStructure::scriptValue() const
 {
     return d->_value;
