@@ -29,29 +29,30 @@
 #include <Document.h>
 #include <DataStructureBackendManager.h>
 #include <DocumentManager.h>
+#include <KDebug>
 
 using namespace Rocs;
 
 TestLinkedListStructure::TestLinkedListStructure()
 {
-    QVERIFY(DataStructureBackendManager::self()->backends().count() > 0);
+    QVERIFY(DataStructureBackendManager::self().backends().count() > 0);
     DocumentManager::self().addDocument(new Document("test"));
 }
 
 void TestLinkedListStructure::listModificationTest()
 {
     // test for the basic properties of the list structure
-    DataStructureBackendManager::self()->setBackend("LinkedList");
+    DataStructureBackendManager::self().setBackend("LinkedList");
     DataStructurePtr ds = DocumentManager::self().activeDocument()->addDataStructure("AddDeleteTest");
     QList< boost::shared_ptr<ListNode> > dataList;
 
     // add one node
-    dataList.append(boost::static_pointer_cast<ListNode>(ds->addData(QString(1))));
+    dataList.append(boost::static_pointer_cast<ListNode>(ds->createData(QString(1),0)));
     QVERIFY2(!dataList.at(0)->next(), "ERROR: no next node expected");
 
     // add node, by this automatically add next pointer of last node in list
-    dataList.append(boost::static_pointer_cast<ListNode>(ds->addData(QString(2))));
-    QVERIFY2(dataList.at(0)->next()->identifier()==dataList.at(1)->identifier(), "ERROR: wrong next node");
+    dataList.append(boost::static_pointer_cast<ListNode>(ds->createData(QString(2),0)));
+    QVERIFY2(dataList.at(1)->next()->identifier() == dataList.at(0)->identifier(), "ERROR: wrong next node");
 
     // remove all data elements
     foreach(DataPtr data, dataList) {
@@ -59,7 +60,7 @@ void TestLinkedListStructure::listModificationTest()
     }
     boost::shared_ptr<Rocs::ListStructure> listStructure = boost::dynamic_pointer_cast<Rocs::ListStructure>(ds);
 
-    QVERIFY2(ds->dataList().size() == 0, "ERROR: Not all data elements were deleted");
+    QVERIFY2(ds->dataList(0).size() == 0, "ERROR: Not all data elements were deleted");
 }
 
 
