@@ -31,7 +31,7 @@
 #include <KPushButton>
 #include <KComboBox>
 #include "DataStructureBackendManager.h"
-#include <DataStructurePluginInterface.h>
+#include <DataStructureBackendInterface.h>
 
 DataStructurePage::DataStructurePage(QWidget* parent)
     : QWidget(parent)
@@ -54,7 +54,7 @@ void DataStructurePage::setDataStructure(DataStructurePtr dataStructure)
     ui->dataStructurePlugin->setDisabled(true);
 
     if (!ui->pluginExtraProperties->layout()) {
-        QLayout * layout = DataStructureBackendManager::self()->dataStructureExtraProperties(dataStructure, ui->pluginExtraProperties);
+        QLayout *layout = DataStructureBackendManager::self().dataStructureExtraProperties(dataStructure, ui->pluginExtraProperties);
         ui->pluginExtraProperties->setLayout(layout);
     }
 
